@@ -18,12 +18,12 @@
 #ifndef ABSTRACTFILELISTENER_H
 #define ABSTRACTFILELISTENER_H
 
-#include "notificationitem.h"
-
 #include <QObject>
 #include <QVector>
 #include <QString>
 
+#include "datatypes.h"
+
 #include <memory>
 
 class AbstractFileListenerPrivate;
@@ -53,7 +53,7 @@ Q_SIGNALS:
 
     void databaseInterfaceChanged();
 
-    void newTrackFile(const MusicAudioTrack &newTrack);
+    void newTrackFile(const DataTypes::TrackDataType &newTrack);
 
     void indexingStarted();
 
@@ -63,10 +63,6 @@ Q_SIGNALS:
 
     void clearDatabase();
 
-    void newNotification(NotificationItem notification);
-
-    void closeNotification(QString notificationId);
-
 public Q_SLOTS:
 
     void setDatabaseInterface(DatabaseInterface* databaseInterface);
@@ -81,8 +77,6 @@ protected:
 
     void setFileListing(AbstractFileListing *fileIndexer);
 
-    NotificationItem& currentNotification();
-
 private:
 
     std::unique_ptr<AbstractFileListenerPrivate> d;
