@@ -25,6 +25,7 @@
 
 #if defined KF5Declarative_FOUND && KF5Declarative_FOUND
 #include <KDeclarative/KDeclarative>
+#include <KQuickAddons/QtQuickSettings>
 #endif
 
 #include <KI18n/KLocalizedString>
@@ -94,6 +95,10 @@ int main(int argc, char *argv[])
 
     QApplication app(argc, argv);
 
+#if defined KF5Declarative_FOUND && KF5Declarative_FOUND
+    KQuickAddons::QtQuickSettings::init();
+#endif
+
     KLocalizedString::setApplicationDomain("elisa");
 
 #if defined Qt5AndroidExtras_FOUND && Qt5AndroidExtras_FOUND
