@@ -17,7 +17,7 @@
 
 #include "alltracksproxymodel.h"
 
-#include "databaseinterface.h"
+#include "datatypes.h"
 
 #include <QReadLocker>
 #include <QtConcurrentRun>
@@ -25,7 +25,6 @@
 AllTracksProxyModel::AllTracksProxyModel(QObject *parent) : AbstractMediaProxyModel(parent)
 {
     setSortCaseSensitivity(Qt::CaseInsensitive);
-    sortModel(Qt::AscendingOrder);
 }
 
 AllTracksProxyModel::~AllTracksProxyModel() = default;
@@ -34,35 +33,22 @@ bool AllTracksProxyModel::filterAcceptsRow(int source_row, const QModelIndex &so
 {
     bool result = false;
 
-    for (int column = 0, columnCount = sourceModel()->columnCount(source_parent); column < columnCount; ++column) {
-        auto currentIndex = sourceModel()->index(source_row, column, source_parent);
+    auto currentIndex = sourceModel()->index(source_row, 0, source_parent);
 
-        const auto &titleValue = sourceModel()->data(currentIndex, Qt::DisplayRole).toString();
-        const auto &artistValue = sourceModel()->data(currentIndex, DatabaseInterface::ColumnsRoles::ArtistRole).toString();
-        const auto maximumRatingValue = sourceModel()->data(currentIndex, DatabaseInterface::ColumnsRoles::RatingRole).toInt();
+    const auto &titleValue = sourceModel()->data(currentIndex, Qt::DisplayRole).toString();
+    const auto &artistValue = sourceModel()->data(currentIndex, DataTypes::ColumnsRoles::ArtistRole).toString();
+    const auto maximumRatingValue = sourceModel()->data(currentIndex, DataTypes::ColumnsRoles::RatingRole).toInt();
 
-        if (maximumRatingValue < mFilterRating) {
-            result = false;
-            continue;
-        }
+    if (maximumRatingValue < mFilterRating) {
+        return result;
+    }
 
-        if (mFilterExpression.match(titleValue).hasMatch()) {
-            result = true;
-            continue;
-        }
+    if (mFilterExpression.match(titleValue).hasMatch()) {
+        result = true;
+    }
 
-        if (mFilterExpression.match(artistValue).hasMatch()) {
-            result = true;
-            continue;
-        }
-
-        if (result) {
-            continue;
-        }
-
-        if (!result) {
-            break;
-        }
+    if (mFilterExpression.match(artistValue).hasMatch()) {
+        result = true;
     }
 
     return result;
@@ -73,12 +59,12 @@ void AllTracksProxyModel::genericEnqueueToPlayList(ElisaUtils::PlayListEnqueueMo
 {
     QtConcurrent::run(&mThreadPool, [=] () {
         QReadLocker locker(&mDataLock);
-        auto allTracks = ElisaUtils::EntryDataList();
+        auto allTracks = ElisaUtils::EntryDataList{};
         allTracks.reserve(rowCount());
         for (int rowIndex = 0, maxRowCount = rowCount(); rowIndex < maxRowCount; ++rowIndex) {
             auto currentIndex = index(rowIndex, 0);
-            allTracks.push_back(ElisaUtils::EntryData{data(currentIndex, DatabaseInterface::ColumnsRoles::DatabaseIdRole).toULongLong(),
-                                 data(currentIndex, DatabaseInterface::ColumnsRoles::TitleRole).toString()});
+            allTracks.push_back(ElisaUtils::EntryData{data(currentIndex, DataTypes::ColumnsRoles::DatabaseIdRole).toULongLong(),
+                                                      data(currentIndex, DataTypes::ColumnsRoles::TitleRole).toString(), {}});
         }
         Q_EMIT entriesToEnqueue(allTracks, ElisaUtils::Track, enqueueMode, triggerPlay);
     });
