@@ -21,7 +21,7 @@
 
 #include <KI18n/KLocalizedString>
 
-#include <QtConcurrent/QtConcurrentRun>
+#include <QtConcurrentRun>
 
 TrackMetadataModel::TrackMetadataModel(QObject *parent)
     : QAbstractListModel(parent)
@@ -55,163 +55,214 @@ QVariant TrackMetadataModel::data(const QModelIndex &index, int role) const
     switch (role)
     {
     case Qt::DisplayRole:
-        result = mTrackData[currentKey];
+        switch (currentKey)
+        {
+        case DataTypes::TrackNumberRole:
+        {
+            auto trackNumber = mTrackData.trackNumber();
+            if (trackNumber > 0) {
+                result = trackNumber;
+            }
+            break;
+        }
+        case DataTypes::DiscNumberRole:
+        {
+            auto discNumber = mTrackData.discNumber();
+            if (discNumber > 0) {
+                result = discNumber;
+            }
+            break;
+        }
+        case DataTypes::ChannelsRole:
+        {
+            auto channels = mTrackData.channels();
+            if (channels > 0) {
+                result = channels;
+            }
+            break;
+        }
+        case DataTypes::BitRateRole:
+        {
+            auto bitRate = mTrackData.bitRate();
+            if (bitRate > 0) {
+                result = bitRate;
+            }
+            break;
+        }
+        case DataTypes::SampleRateRole:
+        {
+            auto sampleRate = mTrackData.sampleRate();
+            if (sampleRate > 0) {
+                result = sampleRate;
+            }
+            break;
+        }
+        default:
+            result = mTrackData[currentKey];
+            break;
+        }
         break;
     case ItemNameRole:
         switch (currentKey)
         {
-        case DatabaseInterface::TitleRole:
+        case DataTypes::TitleRole:
             result = i18nc("Track title for track metadata view", "Title");
             break;
-        case DatabaseInterface::DurationRole:
+        case DataTypes::DurationRole:
             result = i18nc("Duration label for track metadata view", "Duration");
             break;
-        case DatabaseInterface::ArtistRole:
+        case DataTypes::ArtistRole:
             result = i18nc("Track artist for track metadata view", "Artist");
             break;
-        case DatabaseInterface::AlbumRole:
+        case DataTypes::AlbumRole:
             result = i18nc("Album name for track metadata view", "Album");
             break;
-        case DatabaseInterface::AlbumArtistRole:
+        case DataTypes::AlbumArtistRole:
             result = i18nc("Album artist for track metadata view", "Album Artist");
             break;
-        case DatabaseInterface::TrackNumberRole:
+        case DataTypes::TrackNumberRole:
             result = i18nc("Track number for track metadata view", "Track Number");
             break;
-        case DatabaseInterface::DiscNumberRole:
+        case DataTypes::DiscNumberRole:
             result = i18nc("Disc number for track metadata view", "Disc Number");
             break;
-        case DatabaseInterface::RatingRole:
+        case DataTypes::RatingRole:
             result = i18nc("Rating label for information panel", "Rating");
             break;
-        case DatabaseInterface::GenreRole:
+        case DataTypes::GenreRole:
             result = i18nc("Genre label for track metadata view", "Genre");
             break;
-        case DatabaseInterface::LyricistRole:
+        case DataTypes::LyricistRole:
             result = i18nc("Lyricist label for track metadata view", "Lyricist");
             break;
-        case DatabaseInterface::ComposerRole:
+        case DataTypes::ComposerRole:
             result = i18nc("Composer name for track metadata view", "Composer");
             break;
-        case DatabaseInterface::CommentRole:
+        case DataTypes::CommentRole:
             result = i18nc("Comment label for track metadata view", "Comment");
             break;
-        case DatabaseInterface::YearRole:
+        case DataTypes::YearRole:
             result = i18nc("Year label for track metadata view", "Year");
             break;
-        case DatabaseInterface::ChannelsRole:
+        case DataTypes::ChannelsRole:
             result = i18nc("Channels label for track metadata view", "Channels");
             break;
-        case DatabaseInterface::BitRateRole:
+        case DataTypes::BitRateRole:
             result = i18nc("Bit rate label for track metadata view", "Bit Rate");
             break;
-        case DatabaseInterface::SampleRateRole:
+        case DataTypes::SampleRateRole:
             result = i18nc("Sample Rate label for track metadata view", "Sample Rate");
             break;
-        case DatabaseInterface::LastPlayDate:
+        case DataTypes::LastPlayDate:
             result = i18nc("Last play date label for track metadata view", "Last played");
             break;
-        case DatabaseInterface::PlayCounter:
+        case DataTypes::PlayCounter:
             result = i18nc("Play counter label for track metadata view", "Play count");
             break;
-        case DatabaseInterface::LyricsRole:
+        case DataTypes::LyricsRole:
             result = i18nc("Lyrics label for track metadata view", "Lyrics");
             break;
-        case DatabaseInterface::SecondaryTextRole:
-        case DatabaseInterface::ImageUrlRole:
-        case DatabaseInterface::ShadowForImageRole:
-        case DatabaseInterface::ChildModelRole:
-        case DatabaseInterface::StringDurationRole:
-        case DatabaseInterface::MilliSecondsDurationRole:
-        case DatabaseInterface::AllArtistsRole:
-        case DatabaseInterface::HighestTrackRating:
-        case DatabaseInterface::ResourceRole:
-        case DatabaseInterface::IdRole:
-        case DatabaseInterface::DatabaseIdRole:
-        case DatabaseInterface::IsSingleDiscAlbumRole:
-        case DatabaseInterface::ContainerDataRole:
-        case DatabaseInterface::IsPartialDataRole:
-        case DatabaseInterface::AlbumIdRole:
-        case DatabaseInterface::HasEmbeddedCover:
-        case DatabaseInterface::FileModificationTime:
-        case DatabaseInterface::FirstPlayDate:
-        case DatabaseInterface::PlayFrequency:
-        case DatabaseInterface::ElementTypeRole:
+        case DataTypes::ResourceRole:
+            result = i18nc("Radio HTTP address for radio metadata view", "Stream Http Address");
+            break;
+        case DataTypes::SecondaryTextRole:
+        case DataTypes::ImageUrlRole:
+        case DataTypes::ShadowForImageRole:
+        case DataTypes::ChildModelRole:
+        case DataTypes::StringDurationRole:
+        case DataTypes::IsValidAlbumArtistRole:
+        case DataTypes::AllArtistsRole:
+        case DataTypes::HighestTrackRating:
+        case DataTypes::IdRole:
+        case DataTypes::ParentIdRole:
+        case DataTypes::DatabaseIdRole:
+        case DataTypes::IsSingleDiscAlbumRole:
+        case DataTypes::ContainerDataRole:
+        case DataTypes::IsPartialDataRole:
+        case DataTypes::AlbumIdRole:
+        case DataTypes::HasEmbeddedCover:
+        case DataTypes::FileModificationTime:
+        case DataTypes::FirstPlayDate:
+        case DataTypes::PlayFrequency:
+        case DataTypes::ElementTypeRole:
             break;
         }
         break;
     case ItemTypeRole:
         switch (currentKey)
         {
-        case DatabaseInterface::TitleRole:
+        case DataTypes::TitleRole:
             result = TextEntry;
             break;
-        case DatabaseInterface::ArtistRole:
+        case DataTypes::ResourceRole:
             result = TextEntry;
             break;
-        case DatabaseInterface::AlbumRole:
+        case DataTypes::ArtistRole:
             result = TextEntry;
             break;
-        case DatabaseInterface::AlbumArtistRole:
+        case DataTypes::AlbumRole:
             result = TextEntry;
             break;
-        case DatabaseInterface::TrackNumberRole:
+        case DataTypes::AlbumArtistRole:
+            result = TextEntry;
+            break;
+        case DataTypes::TrackNumberRole:
             result = IntegerEntry;
             break;
-        case DatabaseInterface::DiscNumberRole:
+        case DataTypes::DiscNumberRole:
             result = IntegerEntry;
             break;
-        case DatabaseInterface::RatingRole:
+        case DataTypes::RatingRole:
             result = RatingEntry;
             break;
-        case DatabaseInterface::GenreRole:
+        case DataTypes::GenreRole:
             result = TextEntry;
             break;
-        case DatabaseInterface::LyricistRole:
+        case DataTypes::LyricistRole:
             result = TextEntry;
             break;
-        case DatabaseInterface::ComposerRole:
+        case DataTypes::ComposerRole:
             result = TextEntry;
             break;
-        case DatabaseInterface::CommentRole:
+        case DataTypes::CommentRole:
             result = TextEntry;
             break;
-        case DatabaseInterface::YearRole:
+        case DataTypes::YearRole:
             result = IntegerEntry;
             break;
-        case DatabaseInterface::LastPlayDate:
+        case DataTypes::LastPlayDate:
             result = DateEntry;
             break;
-        case DatabaseInterface::PlayCounter:
+        case DataTypes::PlayCounter:
             result = IntegerEntry;
             break;
-        case DatabaseInterface::LyricsRole:
+        case DataTypes::LyricsRole:
             result = LongTextEntry;
             break;
-        case DatabaseInterface::DurationRole:
-        case DatabaseInterface::SampleRateRole:
-        case DatabaseInterface::BitRateRole:
-        case DatabaseInterface::ChannelsRole:
-        case DatabaseInterface::SecondaryTextRole:
-        case DatabaseInterface::ImageUrlRole:
-        case DatabaseInterface::ShadowForImageRole:
-        case DatabaseInterface::ChildModelRole:
-        case DatabaseInterface::StringDurationRole:
-        case DatabaseInterface::MilliSecondsDurationRole:
-        case DatabaseInterface::AllArtistsRole:
-        case DatabaseInterface::HighestTrackRating:
-        case DatabaseInterface::ResourceRole:
-        case DatabaseInterface::IdRole:
-        case DatabaseInterface::DatabaseIdRole:
-        case DatabaseInterface::IsSingleDiscAlbumRole:
-        case DatabaseInterface::ContainerDataRole:
-        case DatabaseInterface::IsPartialDataRole:
-        case DatabaseInterface::AlbumIdRole:
-        case DatabaseInterface::HasEmbeddedCover:
-        case DatabaseInterface::FileModificationTime:
-        case DatabaseInterface::FirstPlayDate:
-        case DatabaseInterface::PlayFrequency:
-        case DatabaseInterface::ElementTypeRole:
+        case DataTypes::DurationRole:
+        case DataTypes::SampleRateRole:
+        case DataTypes::BitRateRole:
+        case DataTypes::ChannelsRole:
+        case DataTypes::SecondaryTextRole:
+        case DataTypes::ImageUrlRole:
+        case DataTypes::ShadowForImageRole:
+        case DataTypes::ChildModelRole:
+        case DataTypes::StringDurationRole:
+        case DataTypes::IsValidAlbumArtistRole:
+        case DataTypes::AllArtistsRole:
+        case DataTypes::HighestTrackRating:
+        case DataTypes::IdRole:
+        case DataTypes::ParentIdRole:
+        case DataTypes::DatabaseIdRole:
+        case DataTypes::IsSingleDiscAlbumRole:
+        case DataTypes::ContainerDataRole:
+        case DataTypes::IsPartialDataRole:
+        case DataTypes::AlbumIdRole:
+        case DataTypes::HasEmbeddedCover:
+        case DataTypes::FileModificationTime:
+        case DataTypes::FirstPlayDate:
+        case DataTypes::PlayFrequency:
+        case DataTypes::ElementTypeRole:
             break;
         }
         break;
@@ -223,7 +274,8 @@ QVariant TrackMetadataModel::data(const QModelIndex &index, int role) const
 bool TrackMetadataModel::setData(const QModelIndex &index, const QVariant &value, int role)
 {
     if (data(index, role) != value) {
-        // FIXME: Implement me!
+        mTrackData[mTrackKeys[index.row()]] = value;
+
         emit dataChanged(index, index, QVector<int>() << role);
         return true;
     }
@@ -245,9 +297,13 @@ QString TrackMetadataModel::fileUrl() const
     return mFileUrl;
 }
 
-const QUrl &TrackMetadataModel::coverUrl() const
+QUrl TrackMetadataModel::coverUrl() const
 {
-    return mCoverImage;
+    if (mCoverImage.isEmpty()) {
+        return QUrl(QStringLiteral("image://icon/media-optical-audio"));
+    } else {
+        return mCoverImage;
+    }
 }
 
 MusicListenersManager *TrackMetadataModel::manager() const
@@ -260,29 +316,39 @@ QString TrackMetadataModel::lyrics() const
     return mFullData[TrackDataType::key_type::LyricsRole].toString();
 }
 
+qulonglong TrackMetadataModel::databaseId() const
+{
+    return mDatabaseId;
+}
+
 void TrackMetadataModel::trackData(const TrackMetadataModel::TrackDataType &trackData)
 {
     if (!mFullData.isEmpty() && trackData.databaseId() != mFullData.databaseId()) {
         return;
     }
 
-    fillDataFromTrackData(trackData);
+    const QList<DataTypes::ColumnsRoles> fieldsForTrack({DataTypes::TitleRole, DataTypes::ArtistRole,
+                                                                 DataTypes::AlbumRole, DataTypes::AlbumArtistRole,
+                                                                 DataTypes::TrackNumberRole, DataTypes::DiscNumberRole,
+                                                                 DataTypes::RatingRole, DataTypes::GenreRole,
+                                                                 DataTypes::LyricistRole, DataTypes::ComposerRole,
+                                                                 DataTypes::CommentRole, DataTypes::YearRole,
+                                                                 DataTypes::LastPlayDate, DataTypes::PlayCounter});
+
+    fillDataFromTrackData(trackData, fieldsForTrack);
 }
 
-void TrackMetadataModel::fillDataFromTrackData(const TrackMetadataModel::TrackDataType &trackData)
+void TrackMetadataModel::fillDataFromTrackData(const TrackMetadataModel::TrackDataType &trackData,
+                                               const QList<DataTypes::ColumnsRoles> &fieldsForTrack)
 {
     beginResetModel();
     mFullData = trackData;
     mTrackData.clear();
     mTrackKeys.clear();
 
-    for (auto role : {DatabaseInterface::TitleRole, DatabaseInterface::ArtistRole, DatabaseInterface::AlbumRole,
-         DatabaseInterface::AlbumArtistRole, DatabaseInterface::TrackNumberRole, DatabaseInterface::DiscNumberRole,
-         DatabaseInterface::RatingRole, DatabaseInterface::GenreRole, DatabaseInterface::LyricistRole,
-         DatabaseInterface::ComposerRole, DatabaseInterface::CommentRole, DatabaseInterface::YearRole,
-         DatabaseInterface::LastPlayDate, DatabaseInterface::PlayCounter}) {
+    for (DataTypes::ColumnsRoles role : fieldsForTrack) {
         if (trackData.constFind(role) != trackData.constEnd()) {
-            if (role == DatabaseInterface::RatingRole) {
+            if (role == DataTypes::RatingRole) {
                 if (trackData[role].toInt() == 0) {
                     continue;
                 }
@@ -297,10 +363,13 @@ void TrackMetadataModel::fillDataFromTrackData(const TrackMetadataModel::TrackDa
 
     fetchLyrics();
 
-    mCoverImage = trackData[DatabaseInterface::ImageUrlRole].toUrl();
+    mDatabaseId = trackData[DataTypes::DatabaseIdRole].toULongLong();
+    Q_EMIT databaseIdChanged();
+
+    mCoverImage = trackData[DataTypes::ImageUrlRole].toUrl();
     Q_EMIT coverUrlChanged();
 
-    auto rawFileUrl = trackData[DatabaseInterface::ResourceRole].toUrl();
+    auto rawFileUrl = trackData[DataTypes::ResourceRole].toUrl();
 
     if (rawFileUrl.isLocalFile()) {
         mFileUrl = rawFileUrl.toLocalFile();
@@ -314,7 +383,7 @@ void TrackMetadataModel::filterDataFromTrackData()
 {
 }
 
-void TrackMetadataModel::removeMetaData(DatabaseInterface::ColumnsRoles metaData)
+void TrackMetadataModel::removeMetaData(DataTypes::ColumnsRoles metaData)
 {
     auto itMetaData = std::find(mTrackKeys.begin(), mTrackKeys.end(), metaData);
     if (itMetaData == mTrackKeys.end()) {
@@ -333,8 +402,8 @@ TrackMetadataModel::TrackDataType::mapped_type TrackMetadataModel::dataFromType(
 void TrackMetadataModel::fillLyricsDataFromTrack()
 {
     beginInsertRows({}, mTrackData.size(), mTrackData.size());
-    mTrackKeys.push_back(DatabaseInterface::LyricsRole);
-    mTrackData[DatabaseInterface::LyricsRole] = mLyricsValueWatcher.result();
+    mTrackKeys.push_back(DataTypes::LyricsRole);
+    mTrackData[DataTypes::LyricsRole] = mLyricsValueWatcher.result();
     endInsertRows();
 }
 
@@ -343,12 +412,24 @@ void TrackMetadataModel::lyricsValueIsReady()
     if (!mLyricsValueWatcher.result().isEmpty()) {
         fillLyricsDataFromTrack();
 
-        mFullData[DatabaseInterface::LyricsRole] = mLyricsValueWatcher.result();
+        mFullData[DataTypes::LyricsRole] = mLyricsValueWatcher.result();
 
         Q_EMIT lyricsChanged();
     }
 }
 
+void TrackMetadataModel::initializeByIdAndUrl(ElisaUtils::PlayListEntryType type, qulonglong databaseId, const QUrl &url)
+{
+    mFullData.clear();
+    mTrackData.clear();
+    mCoverImage.clear();
+    mFileUrl.clear();
+
+    Q_EMIT lyricsChanged();
+
+    Q_EMIT needDataByDatabaseIdAndUrl(type, databaseId, url);
+}
+
 void TrackMetadataModel::initialize(MusicListenersManager *newManager, DatabaseInterface *trackDatabase)
 {
     mManager = newManager;
@@ -356,6 +437,10 @@ void TrackMetadataModel::initialize(MusicListenersManager *newManager, DatabaseI
 
     if (mManager) {
         mDataLoader.setDatabase(mManager->viewDatabase());
+        connect(this, &TrackMetadataModel::needDataByUrl,
+                mManager->tracksListener(), &TracksListener::trackByFileNameInList);
+        connect(mManager->tracksListener(), &TracksListener::trackHasChanged,
+                this, &TrackMetadataModel::trackData);
     } else if (trackDatabase) {
         mDataLoader.setDatabase(trackDatabase);
     }
@@ -364,20 +449,30 @@ void TrackMetadataModel::initialize(MusicListenersManager *newManager, DatabaseI
         mManager->connectModel(&mDataLoader);
     }
 
-    connect(this, &TrackMetadataModel::needDataByDatabaseId,
-            &mDataLoader, &ModelDataLoader::loadDataByDatabaseId);
-    connect(this, &TrackMetadataModel::needDataByFileName,
-            &mDataLoader, &ModelDataLoader::loadDataByFileName);
-    connect(&mDataLoader, &ModelDataLoader::allTrackData,
-            this, &TrackMetadataModel::trackData);
+    connect(this, &TrackMetadataModel::needDataByDatabaseIdAndUrl,
+            &mDataLoader, &ModelDataLoader::loadDataByDatabaseIdAndUrl);
+    connect(this, &TrackMetadataModel::needDataByUrl,
+            &mDataLoader, &ModelDataLoader::loadDataByUrl);
+    connect(this, &TrackMetadataModel::saveRadioData,
+            &mDataLoader, &ModelDataLoader::saveRadioModified);
+    connect(this, &TrackMetadataModel::deleteRadioData,
+            &mDataLoader, &ModelDataLoader::removeRadio);
     connect(&mDataLoader, &ModelDataLoader::trackModified,
             this, &TrackMetadataModel::trackData);
+    connect(&mDataLoader, &ModelDataLoader::allTrackData,
+            this, &TrackMetadataModel::trackData);
+    connect(&mDataLoader, &ModelDataLoader::allRadioData,
+            this, &TrackMetadataModel::radioData);
+    connect(&mDataLoader, &ModelDataLoader::radioAdded,
+            this, &TrackMetadataModel::radioData);
+    connect(&mDataLoader, &ModelDataLoader::radioModified,
+            this, &TrackMetadataModel::radioData);
 }
 
 void TrackMetadataModel::fetchLyrics()
 {
     auto lyricicsValue = QtConcurrent::run(QThreadPool::globalInstance(), [=]() {
-        auto trackData = mFileScanner.scanOneFile(mFullData[DatabaseInterface::ResourceRole].toUrl(), mMimeDatabase);
+        auto trackData = mFileScanner.scanOneFile(mFullData[DataTypes::ResourceRole].toUrl());
         if (!trackData.lyrics().isEmpty()) {
             return trackData.lyrics();
         }
@@ -387,19 +482,40 @@ void TrackMetadataModel::fetchLyrics()
     mLyricsValueWatcher.setFuture(lyricicsValue);
 }
 
-void TrackMetadataModel::initializeByTrackId(qulonglong databaseId)
+void TrackMetadataModel::initializeForNewRadio()
 {
     mFullData.clear();
     mTrackData.clear();
-    mCoverImage.clear();
-    mFileUrl.clear();
 
-    Q_EMIT lyricsChanged();
-
-    Q_EMIT needDataByDatabaseId(ElisaUtils::Track, databaseId);
+    fillDataForNewRadio();
 }
 
-void TrackMetadataModel::initializeByTrackFileName(const QUrl &fileName)
+void TrackMetadataModel::fillDataForNewRadio()
+{
+    beginResetModel();
+    mTrackData.clear();
+    mTrackKeys.clear();
+
+    for (auto role : {
+         DataTypes::TitleRole,
+         DataTypes::ResourceRole,
+         DataTypes::CommentRole,
+         DataTypes::DatabaseIdRole
+
+}) {
+        mTrackKeys.push_back(role);
+        if (role == DataTypes::DatabaseIdRole) {
+            mTrackData[role] = -1;
+        } else {
+            mTrackData[role] = QString();
+        }
+
+    }
+    filterDataFromTrackData();
+    endResetModel();
+}
+
+void TrackMetadataModel::initializeByUrl(ElisaUtils::PlayListEntryType type, const QUrl &url)
 {
     mFullData.clear();
     mTrackData.clear();
@@ -408,7 +524,7 @@ void TrackMetadataModel::initializeByTrackFileName(const QUrl &fileName)
 
     Q_EMIT lyricsChanged();
 
-    Q_EMIT needDataByFileName(ElisaUtils::FileName, fileName);
+    Q_EMIT needDataByUrl(type, url);
 }
 
 void TrackMetadataModel::setManager(MusicListenersManager *newManager)
@@ -421,5 +537,29 @@ void TrackMetadataModel::setDatabase(DatabaseInterface *trackDatabase)
     initialize(nullptr, trackDatabase);
 }
 
+void TrackMetadataModel::saveData()
+{
+    Q_EMIT saveRadioData(mTrackData);
+}
+
+void TrackMetadataModel::deleteRadio()
+{
+    if (mTrackData[DataTypes::DatabaseIdRole]>=0) {
+        Q_EMIT deleteRadioData(mTrackData[DataTypes::DatabaseIdRole].toULongLong());
+    }
+}
+
+void TrackMetadataModel::radioData(const TrackDataType &radiosData)
+{
+    if (!mFullData.isEmpty() && mFullData[DataTypes::DatabaseIdRole].toInt() != -1 &&
+            mFullData.databaseId() != radiosData.databaseId()) {
+        return;
+    }
+
+    const QList<DataTypes::ColumnsRoles> fieldsForTrack({DataTypes::TitleRole, DataTypes::ResourceRole,
+                                                                 DataTypes::CommentRole, DataTypes::DatabaseIdRole});
+
+    fillDataFromTrackData(radiosData, fieldsForTrack);
+}
 
 #include "moc_trackmetadatamodel.cpp"
