@@ -319,7 +319,16 @@ void MusicListenersManager::configChanged()
     currentConfiguration->load();
     currentConfiguration->read();
 
-    const auto &allRootPaths = currentConfiguration->rootPath();
+    //resolve symlinks
+    QStringList allRootPaths;
+    for (const auto &path : currentConfiguration->rootPath()) {
+        QFileInfo pathFileInfo(path);
+        auto directoryPath = pathFileInfo.canonicalFilePath();
+        //directory must always end with a slash
+        directoryPath.append(QLatin1Char('/'));
+        allRootPaths << directoryPath;
+    }
+
     d->mFileListener.setAllRootPaths(allRootPaths);
 
 #if defined KF5Baloo_FOUND && KF5Baloo_FOUND
@@ -391,6 +400,7 @@ void MusicListenersManager::cleanedDatabase()
 {
     d->mImportedTracksCount = 0;
     Q_EMIT importedTracksCountChanged();
+    Q_EMIT clearedDatabase();
 }
 
 void MusicListenersManager::balooAvailabilityChanged()
@@ -484,6 +494,7 @@ void MusicListenersManager::createTracksListener()
         connect(&d->mDatabaseInterface, &DatabaseInterface::trackRemoved, d->mTracksListener.get(), &TracksListener::trackRemoved);
         connect(&d->mDatabaseInterface, &DatabaseInterface::tracksAdded, d->mTracksListener.get(), &TracksListener::tracksAdded);
         connect(&d->mDatabaseInterface, &DatabaseInterface::trackModified, d->mTracksListener.get(), &TracksListener::trackModified);
+        Q_EMIT tracksListenerChanged();
     }
 }
 
