@@ -17,10 +17,11 @@
 
 #include "trackslistener.h"
 
+#include "playListLogging.h"
 #include "databaseinterface.h"
+#include "datatypes.h"
 #include "filescanner.h"
 
-#include <QMimeDatabase>
 #include <QSet>
 #include <QList>
 #include <QDebug>
@@ -34,6 +35,8 @@ public:
 
     QSet<qulonglong> mTracksByIdSet;
 
+    QSet<qulonglong> mRadiosByIdSet;
+
     QList<std::tuple<QString, QString, QString, int, int>> mTracksByNameSet;
 
     QList<QUrl> mTracksByFileNameSet;
@@ -42,8 +45,6 @@ public:
 
     FileScanner mFileScanner;
 
-    QMimeDatabase mMimeDb;
-
 };
 
 TracksListener::TracksListener(DatabaseInterface *database, QObject *parent) : QObject(parent), d(std::make_unique<TracksListenerPrivate>())
@@ -154,53 +155,39 @@ void TracksListener::trackByNameInList(const QVariant &title, const QVariant &ar
     }
 }
 
-void TracksListener::trackByFileNameInList(const QUrl &fileName)
+void TracksListener::trackByFileNameInList(ElisaUtils::PlayListEntryType databaseIdType, const QUrl &fileName)
 {
-    auto newTrackId = d->mDatabase->trackIdFromFileName(fileName);
-    if (newTrackId == 0) {
-        auto newTrack = scanOneFile(fileName);
+    if (fileName.isLocalFile() || fileName.scheme().isEmpty()) {
+        auto newTrackId = d->mDatabase->trackIdFromFileName(fileName);
+        if (newTrackId == 0) {
+            auto newTrack = d->mFileScanner.scanOneFile(fileName);
 
-        if (newTrack.isValid()) {
-            auto oneData = DatabaseInterface::TrackDataType{};
+            if (newTrack.isValid()) {
+                d->mTracksByFileNameSet.push_back(fileName);
 
-            if (!newTrack.title().isEmpty()) {
-                oneData[DatabaseInterface::TrackDataType::key_type::TitleRole] = newTrack.title();
-            } else {
-                const auto &fileUrl = newTrack.resourceURI();
-                oneData[DatabaseInterface::TrackDataType::key_type::TitleRole] = fileUrl.fileName();
+                Q_EMIT trackHasChanged(newTrack);
+                return;
             }
 
-            oneData[DatabaseInterface::TrackDataType::key_type::ArtistRole] = newTrack.artist();
-            oneData[DatabaseInterface::TrackDataType::key_type::AlbumRole] = newTrack.albumName();
-            oneData[DatabaseInterface::TrackDataType::key_type::AlbumIdRole] = newTrack.albumId();
-            oneData[DatabaseInterface::TrackDataType::key_type::TrackNumberRole] = newTrack.trackNumber();
-            oneData[DatabaseInterface::TrackDataType::key_type::DiscNumberRole] = newTrack.discNumber();
-            oneData[DatabaseInterface::TrackDataType::key_type::DurationRole] = newTrack.duration();
-            oneData[DatabaseInterface::TrackDataType::key_type::MilliSecondsDurationRole] = newTrack.duration().msecsSinceStartOfDay();
-            oneData[DatabaseInterface::TrackDataType::key_type::ResourceRole] = newTrack.resourceURI();
-            oneData[DatabaseInterface::TrackDataType::key_type::ImageUrlRole] = newTrack.albumCover();
-
-            Q_EMIT trackHasChanged(oneData);
+            d->mTracksByFileNameSet.push_back(fileName);
 
             return;
         }
+    } else {
+        auto newRadioId = d->mDatabase->radioIdFromFileName(fileName);
+        if (newRadioId) {
+            auto newRadio = d->mDatabase->radioDataFromDatabaseId(newRadioId);
 
-        d->mTracksByFileNameSet.push_back(fileName);
-
-        return;
-    }
-
-    d->mTracksByIdSet.insert(newTrackId);
-
-    auto newTrack = d->mDatabase->trackDataFromDatabaseId(newTrackId);
-
-    if (!newTrack.isEmpty()) {
-        Q_EMIT trackHasChanged({newTrack});
+            if (!newRadio.isEmpty()) {
+                Q_EMIT trackHasChanged({newRadio});
+            }
+        }
     }
 }
 
 void TracksListener::newAlbumInList(qulonglong newDatabaseId, const QString &entryTitle)
 {
+    qCDebug(orgKdeElisaPlayList()) << "TracksListener::newAlbumInList" << newDatabaseId << entryTitle << d->mDatabase->albumData(newDatabaseId);
     Q_EMIT tracksListAdded(newDatabaseId, entryTitle, ElisaUtils::Album, d->mDatabase->albumData(newDatabaseId));
 }
 
@@ -208,6 +195,7 @@ void TracksListener::newEntryInList(qulonglong newDatabaseId,
                                     const QString &entryTitle,
                                     ElisaUtils::PlayListEntryType databaseIdType)
 {
+    qCDebug(orgKdeElisaPlayList()) << "TracksListener::newEntryInList" << newDatabaseId << entryTitle << databaseIdType;
     switch (databaseIdType)
     {
     case ElisaUtils::Track:
@@ -220,11 +208,21 @@ void TracksListener::newEntryInList(qulonglong newDatabaseId,
         }
         break;
     }
+    case ElisaUtils::Radio:
+    {
+        d->mRadiosByIdSet.insert(newDatabaseId);
+
+        auto newRadio = d->mDatabase->radioDataFromDatabaseId(newDatabaseId);
+        if (!newRadio.isEmpty()) {
+            Q_EMIT trackHasChanged(newRadio);
+        }
+        break;
+    }
     case ElisaUtils::Artist:
         newArtistInList(newDatabaseId, entryTitle);
         break;
     case ElisaUtils::FileName:
-        trackByFileNameInList(QUrl::fromLocalFile(entryTitle));
+        newUrlInList(QUrl::fromLocalFile(entryTitle), ElisaUtils::FileName);
         break;
     case ElisaUtils::Album:
         newAlbumInList(newDatabaseId, entryTitle);
@@ -237,6 +235,56 @@ void TracksListener::newEntryInList(qulonglong newDatabaseId,
     }
 }
 
+void TracksListener::newUrlInList(const QUrl &entryUrl, ElisaUtils::PlayListEntryType databaseIdType)
+{
+    switch (databaseIdType)
+    {
+    case ElisaUtils::Track:
+    case ElisaUtils::FileName:
+    {
+        auto newDatabaseId = d->mDatabase->trackIdFromFileName(entryUrl);
+
+        if (!newDatabaseId)
+        {
+            trackByFileNameInList(databaseIdType, entryUrl);
+            return;
+        }
+
+        d->mTracksByIdSet.insert(newDatabaseId);
+
+        auto newTrack = d->mDatabase->trackDataFromDatabaseIdAndUrl(newDatabaseId, entryUrl);
+        if (!newTrack.isEmpty()) {
+            Q_EMIT trackHasChanged(newTrack);
+        }
+        break;
+    }
+    case ElisaUtils::Radio:
+    {
+        auto newDatabaseId = d->mDatabase->radioIdFromFileName(entryUrl);
+
+        if (!newDatabaseId)
+        {
+            return;
+        }
+
+        d->mRadiosByIdSet.insert(newDatabaseId);
+
+        auto newRadio = d->mDatabase->radioDataFromDatabaseId(newDatabaseId);
+        if (!newRadio.isEmpty()) {
+            Q_EMIT trackHasChanged(newRadio);
+        }
+        break;
+    }
+    case ElisaUtils::Artist:
+    case ElisaUtils::Album:
+    case ElisaUtils::Lyricist:
+    case ElisaUtils::Composer:
+    case ElisaUtils::Genre:
+    case ElisaUtils::Unknown:
+        break;
+    }
+}
+
 void TracksListener::newArtistInList(qulonglong newDatabaseId, const QString &artist)
 {
     auto newTracks = d->mDatabase->tracksDataFromAuthor(artist);
@@ -251,10 +299,4 @@ void TracksListener::newArtistInList(qulonglong newDatabaseId, const QString &ar
     Q_EMIT tracksListAdded(newDatabaseId, artist, ElisaUtils::Artist, newTracks);
 }
 
-MusicAudioTrack TracksListener::scanOneFile(const QUrl &scanFile)
-{
-    return d->mFileScanner.scanOneFile(scanFile, d->mMimeDb);
-}
-
-
 #include "moc_trackslistener.cpp"
